/*
==========================================================================
				This file is for LKK(IT6251 LVDs->eDP)
==========================================================================
*/
//#include <stdio.h>
//#include <stdlib.h>
//#include <stdarg.h>
//#include <string.h>
//#include "uart.h"
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/string.h>

#include "IT6251.h"
#include "ddk768_reg.h"
#include "ddk768_chip.h"
#include "ddk768_power.h"
#include "ddk768_timer.h"
#include "ddk768_swi2c.h"
#include "ddk768_help.h"

/* IT6251 InitTable */
typedef struct {
    BYTE addr;
    BYTE offset;
    BYTE andmask;
    BYTE ormask;
} I2C_REG_VALUE;

static I2C_REG_VALUE IT6251_InitTable[] = {
    {DP_I2C_ADDR, 0x05, 0xFF, 0xff}, // DP SW Reset
    {DP_I2C_ADDR, 0x05, 0xFF, 0x00},
    {DP_I2C_ADDR, 0xfd, 0xFF, LVDS_I2C_ADDR},
    {DP_I2C_ADDR, 0xfe, 0xFF, 0x01},

    // LVDS Configuration
    {LVDS_I2C_ADDR, 0x05, 0xFF, 0xff}, // LVDS SW Reset
    {LVDS_I2C_ADDR, 0x05, 0xFF, 0x00},
    {LVDS_I2C_ADDR, 0x3b, 0xFF, 0x42},
    {LVDS_I2C_ADDR, 0x3b, 0xFF, 0x43},
    {LVDS_I2C_ADDR, 0x3c, 0xFF, 0x08 | LVDS_DESSC},
    {LVDS_I2C_ADDR, 0x0b, 0xFF, 0x88},

    {LVDS_I2C_ADDR, 0x2c, 0xFF, LVDS_DESSC_SETTING | LVDS_MAP | LVDS_BPP},

    {LVDS_I2C_ADDR, 0x32, 0xFF, 0x04},
    {LVDS_I2C_ADDR, 0x35, 0xFF, 0xe0},
    {LVDS_I2C_ADDR, 0x05, 0xFF, 0x02},
    {LVDS_I2C_ADDR, 0x05, 0xFF, 0x00},

    {DP_I2C_ADDR, 0x09, 0xFF, 0x07}, // Enable HPD_IRQ, HPD_CHG, VIDSTABLE
    {DP_I2C_ADDR, 0x0A, 0xFF, 0x00},
    {DP_I2C_ADDR, 0x0B, 0xFF, 0x00},
    {DP_I2C_ADDR, 0x06, 0xFF, 0xFF}, // Clear all interrupt
    {DP_I2C_ADDR, 0x07, 0xFF, 0xFF}, // Clear all interrupt
    {DP_I2C_ADDR, 0x08, 0xFF, 0xFF}, // Clear all interrupt

    {DP_I2C_ADDR, 0x16, 0x17, DPTX_SSC_SETTING | LANE_COUNT | TRAINING_BITRATE},
    {DP_I2C_ADDR, 0x23, 0xFF, 0x40},
    {DP_I2C_ADDR, 0x5c, 0xFF, 0xf3},
    {DP_I2C_ADDR, 0x5f, 0xFF, 0x06},

#ifdef DUAL_MODE
    {DP_I2C_ADDR, 0x60, 0xFF, B_HALF_PCLK},
    {DP_I2C_ADDR, 0x61, 0xFF, B_DUAL_INPUT},
#else
    {DP_I2C_ADDR, 0x60, 0xFF, 0x00},
    {DP_I2C_ADDR, 0x61, 0xFF, 0x00},
#endif

    {DP_I2C_ADDR, 0x62, 0xFF, DPTX_BPP},

    {DP_I2C_ADDR, 0xa0, 0xFF, 0x00},
    {DP_I2C_ADDR, 0xc9, 0xFF, 0xf5},
    {DP_I2C_ADDR, 0xca, 0xFF, 0x4d},
    {DP_I2C_ADDR, 0xcb, 0xFF, 0x37},
    // {DP_I2C_ADDR,0xce,0xFF,0x60},
    {DP_I2C_ADDR, 0xd3, 0xFF, 0x03},
    {DP_I2C_ADDR, 0xd4, 0xFF, 0x45},
    {DP_I2C_ADDR, 0xe7, 0xFF, 0xa0},
    {DP_I2C_ADDR, 0xe8, 0xFF, 0x33},
    {DP_I2C_ADDR, 0xec, 0xFF, VIC},
    {DP_I2C_ADDR, 0x23, 0xFF, 0x42},
    {DP_I2C_ADDR, 0x24, 0xFF, 0x00},
    {DP_I2C_ADDR, 0x25, 0xFF, 0x00},
    {DP_I2C_ADDR, 0x26, 0xFF, 0x00},
    {DP_I2C_ADDR, 0x2b, 0xFF, 0x00},
    {DP_I2C_ADDR, 0x23, 0xFF, 0x40},
    {DP_I2C_ADDR, 0x17, 0xFF, 0x04},
    {DP_I2C_ADDR, 0x17, 0xFF, 0x01},
};

#define Count_IT6251_InitTable                                                 \
    (sizeof(IT6251_InitTable) / sizeof(I2C_REG_VALUE))

void delayms (volatile unsigned long millisecond)
{
    while (millisecond-- > 0)
    {
        //timer1Delay();
        volatile unsigned long d;
        for (d = 0; d <=0x20cb; d++)
            ;
    }
}

#ifdef DUMP_6251
void dptx_show_vid_info(void)
{
    unsigned short HSyncPol, VSyncPol, InterLaced;
    unsigned short HTotal, HDEW, HFPH, HSYNCW;
    unsigned short VTotal, VDEW, VFPH, VSYNCW;
    unsigned short rddata;
    unsigned short MODE1, MODE2;
    unsigned long PCLK, sum;
    BYTE sysstat,linkstat ;


    sysstat = ddk768_swI2CReadReg(DP_I2C_ADDR,0x0D);
    linkstat = ddk768_swI2CReadReg(DP_I2C_ADDR,0x0E);

    if((linkstat & 0x1F) != 0x10 )
    {
        printk("Link Training has something wrong, reg0E = %02X\n",(int)linkstat) ;
    }
    else
    {
        printk("reg0E = %02X, Link Rate = %s",(int)linkstat,(ddk768_swI2CReadReg(DP_I2C_ADDR,0x0E)&0x80)?"HBR":"LBR") ;
    }

    printk("reg0D = %02X, %s%s%s\n",(int)sysstat,(sysstat&1)?"Interrupt! ":"",(sysstat&2)?"HPD ":"Unplug " ,(sysstat&4)?"Video Stable":"Video Unstable" ) ;

    if( (sysstat & 0x04) ==0 )
    {
        return ;
    }

    rddata = ddk768_swI2CReadReg(DP_I2C_ADDR,0xA0);
    HSyncPol = rddata&0x01;
    VSyncPol = (rddata&0x04)>>2;
    InterLaced = (rddata&0x10)>>4;

    rddata = ddk768_swI2CReadReg(DP_I2C_ADDR,0xA1);
    HTotal = ((ddk768_swI2CReadReg(DP_I2C_ADDR,0xA2)&0x1F)<<8) + rddata;

    //rddata = swI2CReadReg(DP_I2C_ADDR,0xA3);
    //HDES = ((swI2CReadReg(DP_I2C_ADDR,0xA4)&0x03)<<8) + rddata;

    rddata = ddk768_swI2CReadReg(DP_I2C_ADDR,0xA5);
    HDEW = ((ddk768_swI2CReadReg(DP_I2C_ADDR,0xA6)&0x1F)<<8) + rddata;

    rddata = ddk768_swI2CReadReg(DP_I2C_ADDR,0xA7);
    HFPH = ((ddk768_swI2CReadReg(DP_I2C_ADDR,0xA8)&0x03)<<8) + rddata;

    rddata = ddk768_swI2CReadReg(DP_I2C_ADDR,0xA9);
    HSYNCW = ((ddk768_swI2CReadReg(DP_I2C_ADDR,0xAA)&0x03)<<8) + rddata;

    rddata = ddk768_swI2CReadReg(DP_I2C_ADDR,0xAB);
    VTotal = ((ddk768_swI2CReadReg(DP_I2C_ADDR,0xAC)&0x0F)<<8) + rddata;

    //rddata = swI2CReadReg(DP_I2C_ADDR,0xAD);
    //VDES = ((swI2CReadReg(DP_I2C_ADDR,0xAE)&0x01)<<8) + rddata;

    rddata = ddk768_swI2CReadReg(DP_I2C_ADDR,0xAF);
    VDEW = ((ddk768_swI2CReadReg(DP_I2C_ADDR,0xB0)&0x0F)<<8) + rddata;

    rddata = ddk768_swI2CReadReg(DP_I2C_ADDR,0xB1);
    VFPH = ((ddk768_swI2CReadReg(DP_I2C_ADDR,0xB2)&0x01)<<8) + rddata;

    rddata = ddk768_swI2CReadReg(DP_I2C_ADDR,0xB3);
    VSYNCW = ((ddk768_swI2CReadReg(DP_I2C_ADDR,0xB4)&0x01)<<8) + rddata;

    MODE1 = (ddk768_swI2CReadReg(DP_I2C_ADDR,0x60)&0x02)>>1;
    MODE2 = (ddk768_swI2CReadReg(DP_I2C_ADDR,0x61)&0x04)>>2;

    printk("DUAL MODE: %d, %d\n",MODE1,MODE2) ;
    printk("%4dx%4d",HDEW,VDEW) ;
    printk("(%4dx%4d)\n",HTotal,VTotal) ;
    printk("(%d,%d,%d),", HFPH, HSYNCW  , HTotal - HDEW - HFPH - HSYNCW);
    printk("(%d,%d,%d),",VFPH,VSYNCW,VTotal - VDEW - VFPH - VSYNCW) ;
    printk("%s",(InterLaced&0x01)?"INTERLACED":"PROGRESS") ;
    printk("{H%s,V%s} ",(HSyncPol&0x01)?"-":"+",(VSyncPol&0x01)?"-":"+") ;

    rddata = ddk768_swI2CReadReg(DP_I2C_ADDR,0x12) ;
    rddata |= 0x80 ;
    ddk768_swI2CWriteReg(DP_I2C_ADDR,0x12, (BYTE)rddata) ;
    rddata &= 0x7f ;
    delayms(1) ;
    ddk768_swI2CWriteReg(DP_I2C_ADDR,0x12, (BYTE)rddata) ;

    sum = ddk768_swI2CReadReg(DP_I2C_ADDR,0x13);
    sum = ((ddk768_swI2CReadReg(DP_I2C_ADDR,0x14)&0x0F)<<8)+rddata;
    PCLK = (13500L*2048L)/sum;
    printk("PCLK = %ldHz, xCnt = %ld\n",PCLK,sum);
}
#endif

int IT6251_probe(void)
{
#define IT6251_VENDOR_ID_LOW 0x00
#define IT6251_VENDOR_ID_HIGH 0x01
#define IT6251_DEVICE_ID_LOW 0x02
#define IT6251_DEVICE_ID_HIGH 0x03
    const int max_tries = 2;
    unsigned char vendor_id_lo, vendor_id_hi, device_id_lo, device_id_hi;
    int i = 0;

    ddk768_swI2CInit(DP_SWI2C_SCL, DP_SWI2C_SDA);
    
    for (i = 0; i < max_tries; i++) {
        vendor_id_lo = ddk768_swI2CReadReg(DP_I2C_ADDR, IT6251_VENDOR_ID_LOW);
        vendor_id_hi = ddk768_swI2CReadReg(DP_I2C_ADDR, IT6251_VENDOR_ID_HIGH);
        device_id_lo = ddk768_swI2CReadReg(DP_I2C_ADDR, IT6251_DEVICE_ID_LOW);
        device_id_hi = ddk768_swI2CReadReg(DP_I2C_ADDR, IT6251_DEVICE_ID_HIGH);

        if ((vendor_id_lo != 0xff) && (vendor_id_hi != 0xff) &&
            (device_id_lo != 0xff) && (device_id_hi != 0xff))
            break;
        usleep_range(100000, 200000);
    }

    if ((vendor_id_lo == 0xff) || (vendor_id_hi == 0xff) || (device_id_lo == 0xff) ||
        (device_id_hi == 0xff)) {
        return -1;
    }

    return 0;
}

static void dptx_auxwait(void) {
    int auxwaitcnt;

    for (auxwaitcnt = 0; auxwaitcnt < 200; auxwaitcnt++) {
        if ((ddk768_swI2CReadReg(DP_I2C_ADDR, 0x2b) & 0x20) != 0x20)
            return;
        usleep_range(10000, 11000);
    }

    printk("ERROR: AUX Channel Hang => ");

    switch (ddk768_swI2CReadReg(DP_I2C_ADDR, 0x9f) & 0x03) {
    case 0:
        printk("No Error !!!\n");
        break;
    case 1:
        printk("Defer > 7 Times !!!\n");
        break;
    case 2:
        printk("Receive NAck Response !!!\n");
        break;
    default:
        printk("TimeOut !!!\n");
        break;
    }
}

BYTE dptx_AUX_R_EDID(BYTE NoSegW, BYTE block, BYTE offset) {

    ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x23,
                         (NoSegW << 6) + 0x03);       // Enable PC Aux Access
    ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x23,
                         (NoSegW << 6) + 0x82); // EDID FIFO Clr
    ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x24,
                         (block % 2) * 128 + offset); // Start Address[7:0]
    ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x25, block / 256); // Start Address[15:8]
    // dptxwr(0x25, 0); // Start Address[15:8]
    ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x26,
                         0xF0);       // WriteNum[3:0]+StartAdr[19:16]
    ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x2B, 0x0b); // Aux EDID Read Fire
    dptx_auxwait();

    if (ddk768_swI2CReadReg(DP_I2C_ADDR, 0x07) & 0x01) {
#ifdef DEBUG_DP_TX
        printk("AUX Channel PC Request Fail Interrupt ...\n");
#endif
        //           //DPTX_DEBUG_PRINTF(("Repeat the same EDID read request:
        //           offset=%d to %d\n", offset, offset+7));

        switch ((ddk768_swI2CReadReg(DP_I2C_ADDR, 0x9F) & 0xC0) >> 6) {
        case 0:
#ifdef DEBUG_DP_TX
            printk(("No Error !!!\n"));
#endif
            return 0;
            break;
        case 1:
#ifdef DEBUG_DP_TX
            printk(("Defer > 7 Times !!!\n"));
#endif
            return 0;
            break;

        case 2:
#ifdef DEBUG_DP_TX
            printk(("Receive NAck Response !!!\n"));
#endif
            return 0;
            break;
        default:
#ifdef DEBUG_DP_TX
            printk(("TimeOut !!!\n"));
#endif
            return 0;
            break;
        }
    }
    return 1;
}

int dptxset(BYTE offset, BYTE mask, BYTE value) {
    unsigned char uc;
    uc = ddk768_swI2CReadReg(DP_I2C_ADDR, offset);
    uc &= ~mask;
    uc |= value & mask;
    return ddk768_swI2CWriteReg(DP_I2C_ADDR, offset, uc);
}

int CheckEDIDHeader(void)
{
    uint8_t offset;
    uint8_t pEDIDHeader[16] = {0};

    dptxset(0x05, 0x08, 0x08);
    dptxset(0x05, 0x08, 0x00);

    dptxset(0x07, 0x01, 0x01); // Clear PCRqNAck Interrupt
    dptxset(0x0A, 0x01, 0x01); // Enable PCRqNAck Interrupt

    dptx_AUX_R_EDID(1, 0, 0);
    for (offset = 0; offset < 0x8; offset++) {
        pEDIDHeader[offset] = ddk768_swI2CReadReg(DP_I2C_ADDR, 0x2F);
    }

    ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x23,
                         (1 << 6) + 0x00); // Disable PC Aux Access

    if (pEDIDHeader[0] == 0 && pEDIDHeader[1] == 0xff && pEDIDHeader[2] == 0xff &&
        pEDIDHeader[3] == 0xff && pEDIDHeader[4] == 0xff && pEDIDHeader[5] == 0xff &&
        pEDIDHeader[6] == 0xff && pEDIDHeader[7] == 0)
        return 0;
    else
        return -1;
}

BYTE GetEDIDBlock(BYTE *pEDIDBuff, BYTE blockno)
{
    uint16_t i;
    uint8_t offset, checksum;

    if (pEDIDBuff == NULL) {
        return 0xFF;
    }

    dptxset(0x05, 0x08, 0x08);
    dptxset(0x05, 0x08, 0x00);

    dptxset(0x07, 0x01, 0x01); // Clear PCRqNAck Interrupt
    dptxset(0x0A, 0x01, 0x01); // Enable PCRqNAck Interrupt

    for (offset = 0; offset < 0x80; offset += 0x10) {
        dptx_AUX_R_EDID(1, blockno, offset);

        for (i = 0; i < 0x10; i++) {
            *(pEDIDBuff + offset + i) = ddk768_swI2CReadReg(DP_I2C_ADDR, 0x2F);
        }
    }

    ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x23,
                         (1 << 6) + 0x00); // Disable PC Aux Access
    checksum = 0;
    for (i = 0; i < 0x80; i++) {
        checksum += *(pEDIDBuff + i);
        checksum &= 0xFF;
    }

    return checksum;
}

static int dptx_get_edid(uint8_t *edid)
{
    uint8_t blockno,checksum;
    uint8_t *pEDID = edid;

    memset(edid, 0, 0x100);

    ddk768_swI2CInit(DP_SWI2C_SCL, DP_SWI2C_SDA);
    checksum = GetEDIDBlock(pEDID, 0);
    if( checksum != 0 ) {
        return 0;
    }
    blockno = pEDID[0x7E] + 1;
    if (blockno > 1) {
        checksum = GetEDIDBlock(pEDID + 0x80, 1);
		if( checksum != 0 ) {
			return 0;
		}
        return 0x100;
    }
    else {
        return 0x80;
    }
}

int IT6251_Read_EDID(uint8_t *edidBuf)
{
    return dptx_get_edid(edidBuf);
}

int IT6251_Check(void)
{
    /* Enable the GPIO pin 12 as GPIO */
    pokeRegisterDWord(GPIO_MUX, peekRegisterDWord(GPIO_MUX) & ~(1 << 12));
    /* Set GPIO pin 12 as Input */
    pokeRegisterDWord(GPIO_DATA_DIRECTION, peekRegisterDWord(GPIO_DATA_DIRECTION) & ~(1 << 12));
    /* Get the GPIO pin 12 value */
	return (peekRegisterDWord(GPIO_DATA) & (1 << 12));
}

void IT6251_Init(void)
{
    int i ;
    int Loopcount = 5 ;
    unsigned char ucValue ;
#ifdef DUMP_IT6251_REG
    unsigned char ucValue2 ;
#endif
    BYTE linkstat ;
    
	printk("IT6251_Init ++\n");
    ddk768_swI2CInit(DP_SWI2C_SCL, DP_SWI2C_SDA);
    
    for( i = 0 ; i < Count_IT6251_InitTable ; i++ )
    {
        if(IT6251_InitTable[i].andmask != 0xFF )
        {
            ucValue=ddk768_swI2CReadReg(IT6251_InitTable[i].addr,
                        IT6251_InitTable[i].offset );

            ucValue &= ~(IT6251_InitTable[i].andmask) ;
            ucValue |= IT6251_InitTable[i].ormask ;
            ddk768_swI2CWriteReg(IT6251_InitTable[i].addr,
                        IT6251_InitTable[i].offset,
                        ucValue);
#ifdef DUMP_IT6251_REG
            ucValue2=ddk768_swI2CReadReg(IT6251_InitTable[i].addr,
                        IT6251_InitTable[i].offset );
            printk("ADDR:%02X, OFFSET:%02X, Value1: %02X, Value2: %02X\n",(int)IT6251_InitTable[i].addr, (int)IT6251_InitTable[i].offset, (int)ucValue, (int)ucValue2);
#endif
        }
        else
        {
            ddk768_swI2CWriteReg(IT6251_InitTable[i].addr,
                        IT6251_InitTable[i].offset,
                        IT6251_InitTable[i].ormask);
#ifdef DUMP_IT6251_REG
            ucValue2=ddk768_swI2CReadReg(IT6251_InitTable[i].addr,
                        IT6251_InitTable[i].offset );
            printk("ADDR:%02X, OFFSET:%02X, Value1: %02X, Value2: %02X\n",(int)IT6251_InitTable[i].addr, (int)IT6251_InitTable[i].offset, (int)IT6251_InitTable[i].ormask, (int)ucValue2);
#endif
        }
    }
    // 2011/08/01 modified by Alin
    ucValue=ddk768_swI2CReadReg(DP_I2C_ADDR, 0x0D);
    //linkstat=(ucValue&0x02)?TRUE:FALSE ;
// checking timeout, in 500ms
    while(Loopcount)
    {
        for(i=0 ; i <200 ; i ++ )
        {
            delayms(5);
            linkstat = ddk768_swI2CReadReg(DP_I2C_ADDR, 0x0E);
            if((linkstat & 0x1F) == 0x10)
            {
                // normal operation
                // check if the training bitrate same to bitrate
                if(linkstat & 0x80)
                {
                    ucValue = B_HBR ;
                }
                else
                {
                    ucValue = B_LBR ;
                }

                if( ucValue == TRAINING_BITRATE )
                {
                    return ;
                }
                else
                {
                    printk("TARGET BITRATE is %s, link state = %s, not match, retrain.\n",(TRAINING_BITRATE==B_LBR)?"LBR":"HBR",(linkstat&0x80)?"LBR":"HBR") ;
                    // retrain.
                    break ;
                }
            }
        }
        
        if((linkstat & 0x1F)!=0x10)
            printk("Link training fail, reg0E = %02X\n",(int)linkstat);

#ifdef DUMP_6251
        dptx_show_vid_info();
#endif
        // training result fail, retrain
        ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x05, 0x00);
       	ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x17, 0x04);
        ddk768_swI2CWriteReg(DP_I2C_ADDR, 0x17, 0x01);
        Loopcount--;
    }
	printk("IT6251_Init --\n");
}