/******************************************************************************

  File        :  LKK_IT6251.h

  Description :  Register Address definitions of IT6251.

******************************************************************************/
#ifndef _IT6251_H_
#define _IT6251_H_

#ifndef BYTE
typedef unsigned char   BYTE;
#endif

#define DP_I2C_ADDR 0xB8
#define LVDS_I2C_ADDR 0xBC
#define EDID_SEG_WRITE (0 << 6)

#ifndef UNSTABLE_THRESHOLD
	#define UNSTABLE_THRESHOLD 10
#endif

/* LVDS reg2C */
#define B_LVDS_6Bpp  			0
#define B_LVDS_8Bpp  			1
#define B_LVDS_10Bpp  			2
#define B_LVDS_12Bpp  			3
#define B_JEIDA       			0
#define B_VESA 					(1<<4)
#define B_ENABLE_DESSC 			(0<<6)
#define B_DISABLE_DESSC 		(1<<6)
/* LVDS reg3C */
#define B_LVDS_DESSC_ON 		7
#define B_LVDS_DESSC_OFF 		0
/* LVDS reg35 */
#define B_DCBAL 				(1<<4)
/* LVDS reg32 */
#define B_AUTO_DCBAL_DISABLE 	(1<<2)
#define B_AUTO_DCBAL_ENABLE  	(0<<2)

/* DPTX reg16 */
#define B_LBR    				1
#define B_HBR    				0
#define B_4_LANE 				(3<<1)
#define B_2_LANE 				(1<<1)
#define B_1_LANE 				(0<<1)
#define B_SSC_ENABLE   			(1<<4)
#define B_SSC_DISABLE   		(0<<4)
/* DPTX reg60 */
#define B_HALF_PCLK     		(1<<1)
/* DPTX reg61 */
#define B_SINGLE_INPUT  		(0<<2)
#define B_DUAL_INPUT    		(1<<2)
/* DPTX reg62[3:0] */
#define B_DPTXIN_6Bpp   		0
#define B_DPTXIN_8Bpp   		1
#define B_DPTXIN_10Bpp  		2
#define B_DPTXIN_12Bpp  		3
#define AUXWAITTIME     		10 // in ms
/* non-Zero value for CEA setting, check the given input format. */
#define VIC						0

/*if LVDS is map1 , use default. Otherwise, define LVDS_MAP3 */
#define DUAL_MODE
#define LVDS_MAP3
//#define ENABLE_LVDS_DESSC

// #define COLOR_6BPP
#define COLOR_8BPP
// #define COLOR_10BPP
// #define COLOR_12BPP

//#define ENABLE_DPTX_SSC
//#define FORCE_LBR

/* option for lane count */
#ifndef LANE_COUNT
//#define LANE_COUNT  B_4_LANE
#define LANE_COUNT  B_2_LANE
//#define LANE_COUNT  B_1_LANE
#endif


#ifdef FORCE_LBR
#define TRAINING_BITRATE     B_LBR
#else
#define TRAINING_BITRATE     B_HBR
#endif

/* option for SSC */
#ifdef ENABLE_DPTX_SSC
#define DPTX_SSC_SETTING         B_SSC_ENABLE
#else
#define DPTX_SSC_SETTING         B_SSC_DISABLE
#endif

#ifndef UNSTABLE_THRESHOLD
#define UNSTABLE_THRESHOLD 10
#endif

#ifdef LVDS_MAP3
#define LVDS_MAP    B_VESA
#else
#define LVDS_MAP    B_JEIDA
#endif

#ifdef ENABLE_LVDS_DESSC
#define LVDS_DESSC         B_LVDS_DESSC_ON
#define LVDS_DESSC_SETTING B_ENABLE_DESSC
#else
#define LVDS_DESSC         B_LVDS_DESSC_OFF
#define LVDS_DESSC_SETTING B_DISABLE_DESSC
#endif


/* options for colordepth */
#ifdef COLOR_6BPP
#define LVDS_BPP B_LVDS_6Bpp
#define DPTX_BPP B_DPTXIN_6Bpp
#elif defined(COLOR_10BPP)
#define LVDS_BPP B_LVDS_10Bpp
#define DPTX_BPP B_DPTXIN_10Bpp
#elif defined(COLOR_12BPP)
#define LVDS_BPP B_LVDS_12Bpp
#define DPTX_BPP B_DPTXIN_12Bpp
#else // COLOR_8BPP
#define LVDS_BPP B_LVDS_8Bpp
#define DPTX_BPP B_DPTXIN_8Bpp
#endif

#define DUMP_6251
#define DUMP_IT6251_REG

#define PWM_0 				0    //Use PWM0 for Backlight
#define PWM_DIVISOR 		7    //divisor for input clock is 2^7
#define PWM_DIVISOR_VALUE 	128   //divisor for input clock is 2^7 = 128
#define PWM_FREQUENCY 		200 //frequency for input clock is 200Hz
#define PWM_DEFAULTF 		42000000 //default frequency for input clock is 42MHz

#ifdef DUMP_6251
void dptx_show_vid_info(void);
#endif
void delayms(volatile unsigned long);
void IT6251_Init(void);
int IT6251_probe(void);
int IT6251_Read_EDID(uint8_t *edidBuf);
int IT6251_Check(void);

#endif // _IT6251_H_
