#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/uaccess.h>
#include <linux/version.h>

#include "sm768_lvds.h"
#include "smi_dbg.h"

extern int lvds_channel;
mode_parameter_t LVDSModeTbl;

mode_parameter_t ParseString(char *buf)
{
	const char *s = ",";
	char *token;
	mode_parameter_t t;

	token = strsep(&buf, s);
	t.horizontal_total = simple_strtol(token, NULL, 10);
	token = strsep(&buf, s);
	t.horizontal_display_end = simple_strtol(token, NULL, 10);
	token = strsep(&buf, s);
	t.horizontal_sync_start = simple_strtol(token, NULL, 10);
	token = strsep(&buf, s);
	t.horizontal_sync_width = simple_strtol(token, NULL, 10);
	token = strsep(&buf, s);
	if (strcmp(token, "POS") == 0)
		t.horizontal_sync_polarity = POS;
	else
		t.horizontal_sync_polarity = NEG;

	token = strsep(&buf, s);
	t.vertical_total = simple_strtol(token, NULL, 10);
	token = strsep(&buf, s);
	t.vertical_display_end = simple_strtol(token, NULL, 10);
	token = strsep(&buf, s);
	t.vertical_sync_start = simple_strtol(token, NULL, 10);
	token = strsep(&buf, s);
	t.vertical_sync_height = simple_strtol(token, NULL, 10);
	token = strsep(&buf, s);
	if (strcmp(token, "POS") == 0)
		t.vertical_sync_polarity = POS;
	else
		t.vertical_sync_polarity = NEG;

	token = strsep(&buf, s);
	t.pixel_clock = simple_strtol(token, NULL, 10);
	token = strsep(&buf, s);
	t.horizontal_frequency = simple_strtol(token, NULL, 10);
	token = strsep(&buf, s);
	t.vertical_frequency = simple_strtol(token, NULL, 10);

	token = strsep(&buf, s);
	if (strcmp(token, "POS") == 0)
		t.clock_phase_polarity = POS;
	else
		t.clock_phase_polarity = NEG;

	token = strsep(&buf, s);
	lvds_channel = simple_strtol(token, NULL, 10);

	return t;
}

int ReadLVDSConfiguration(void)
{
	struct file *fp;
	loff_t pos = 0;
	static char buf[1024];

	fp = filp_open("/etc/smifb.modes", O_RDONLY, 0644);
	if (IS_ERR(fp))
		return -1;

	kernel_read(fp, buf, 1024, &pos);
	LVDSModeTbl = ParseString(buf);
	dbg_msg("lvds_channel:%d\n", lvds_channel);
#ifdef LVDS_DEBUG
	dbg_msg("LVDSModeTbl:%lu,%lu,%lu,%lu,%s,%lu,%lu,%lu,%lu,%s,%lu,%lu,%lu,%s\n",
		LVDSModeTbl.horizontal_total, LVDSModeTbl.horizontal_display_end,
		LVDSModeTbl.horizontal_sync_start, LVDSModeTbl.horizontal_sync_width,
		LVDSModeTbl.horizontal_sync_polarity == POS ? "POS" : "NEG",
		LVDSModeTbl.vertical_total, LVDSModeTbl.vertical_display_end,
		LVDSModeTbl.vertical_sync_start, LVDSModeTbl.vertical_sync_height,
		LVDSModeTbl.vertical_sync_polarity == POS ? "POS" : "NEG", LVDSModeTbl.pixel_clock,
		LVDSModeTbl.horizontal_frequency, LVDSModeTbl.vertical_frequency,
		LVDSModeTbl.clock_phase_polarity == POS ? "POS" : "NEG");
	dbg_msg("lvds_channel:%d\n", lvds_channel);
#endif
	filp_close(fp, NULL);
	return 0;
}
