#ifndef __SMI_DBG_H__
#define __SMI_DBG_H__
#include <linux/version.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 16, 0)
#include <drm/drm_print.h>
#else
#include <drm/drmP.h>
#endif

#ifndef DEBUG
#define DEBUG 0
#endif
#define PFX "[smi] "
extern int smi_indent;

#if (DEBUG == 0)
#define ENTER()
#define LEAVE(...) return __VA_ARGS__;
#define dbg_msg(fmt, args...)
#define err_msg(fmt, args...) DRM_ERROR(fmt, ##args)
#define war_msg(fmt, args...) DRM_WARN(fmt, ##args)
#define inf_msg(fmt, args...) DRM_INFO(fmt, ##args)

#elif (DEBUG == 1)
/* debug level == 1 */
#define ENTER() printk(KERN_DEBUG PFX "%*c %s\n", smi_indent++, '>', __func__)
#define LEAVE(...)                                                                                 \
	{                                                                                          \
		printk(KERN_DEBUG PFX "%*c %s\n", --smi_indent, '<', __func__);                    \
		return __VA_ARGS__;                                                                \
	}
#define dbg_msg(fmt, args...) printk(KERN_DEBUG "[%s]:" fmt, __func__, ##args)
#define err_msg(fmt, args...) printk(KERN_ERR fmt, ##args)
#define war_msg(fmt, args...) printk(KERN_WARNING fmt, ##args)
#define inf_msg(fmt, args...) printk(KERN_INFO fmt, ##args)

#elif (DEBUG == 2)
/* debug level == 2 */
#define ENTER() printk(KERN_DEBUG PFX "%*c %s\n", smi_indent++, '>', __func__)
#define LEAVE(...)                                                                                 \
	do {                                                                                       \
		printk(KERN_DEBUG PFX "%*c %s(%d)\n", --smi_indent, '<', __func__, __LINE__);      \
		return __VA_ARGS__;                                                                \
	} while (0)
#define dbg_msg(fmt, args...) printk(KERN_DEBUG PFX "[%s]:" fmt, __func__, ##args)
#define err_msg(fmt, args...) printk(KERN_ERR PFX fmt, ##args)
#define war_msg(fmt, args...) printk(KERN_WARNING PFX fmt, ##args)
#define inf_msg(fmt, args...) printk(KERN_INFO PFX fmt, ##args)

#endif
#endif
