#ifndef __SMI_DBG_H__
#define __SMI_DBG_H__
#include <linux/version.h>

#include <drm/drm_print.h>

#ifndef DEBUG
#define DEBUG 0
#endif

extern int smi_indent;

#if (DEBUG == 0)
#define ENTER()
#define LEAVE(...) return __VA_ARGS__;
#define dbg_msg(fmt, args...)

#elif (DEBUG == 1)
/* debug level == 1 */
#define ENTER() DRM_DEBUG_DRIVER("%*c %s\n", smi_indent++, '>', __func__)
#define LEAVE(...)                                                                                 \
	do {                                                                                       \
		DRM_DEBUG_DRIVER("%*c %s\n", --smi_indent, '<', __func__);                         \
		return __VA_ARGS__;                                                                \
	} while (0)
#define dbg_msg(fmt, args...)

#elif (DEBUG == 2)
/* debug level == 2 */
#define ENTER() DRM_INFO("%*c %s\n", smi_indent++, '>', __func__)
#define LEAVE(...)                                                                                 \
	do {                                                                                       \
		DRM_INFO("%*c %s(%d)\n", --smi_indent, '<', __func__, __LINE__);                   \
		return __VA_ARGS__;                                                                \
	} while (0)
#define dbg_msg(fmt, args...) DRM_DEBUG_DRIVER(fmt, ##args)
#endif
#endif