/*
 * Copyright 2016 SiliconMotion Inc.
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License version 2. See the file COPYING in the main
 * directory of this archive for more details.
 *
 */

#ifndef __SMI_DRV_H__
#define __SMI_DRV_H__

#include <linux/version.h>

#define RHEL_VERSION_HIGHER_THAN(a, b) (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(a, b))
#define RHEL_VERSION_LOWER_THAN(a, b) (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(a, b))

#include <drm/drm_edid.h>
#if RHEL_VERSION_HIGHER_THAN(7, 4)
#include <drm/drm_encoder.h>
#endif
#include <drm/drm_fb_helper.h>
#if RHEL_VERSION_HIGHER_THAN(7, 3)
#include <drm/drm_gem.h>
#endif
#include <video/vga.h>
#include <drm/drm_crtc.h>
#if RHEL_VERSION_LOWER_THAN(8, 2)
#include <drm/drmP.h>
#include <drm/ttm/ttm_bo_api.h>
#include <drm/ttm/ttm_bo_driver.h>
#include <drm/ttm/ttm_placement.h>
#include <drm/ttm/ttm_memory.h>
#include <drm/ttm/ttm_module.h>
#else
#include <drm/drm_gem_vram_helper.h>
#if RHEL_VERSION_LOWER_THAN(8, 3)
#include <drm/drm_vram_mm_helper.h>
#endif
#endif

#include "smi_priv.h"
#include "smi_dbg.h"

#define DRIVER_AUTHOR "SiliconMotion"

#define DRIVER_NAME "smifb"
#define DRIVER_DESC "SiliconMotion GPU DRM Driver"
#define DRIVER_DATE "20200509"

#define DRIVER_MAJOR 1
#define DRIVER_MINOR 3
#define DRIVER_PATCHLEVEL 1

#define SMIFB_CONN_LIMIT 3

#define RELEASE_TYPE "Linux DRM Display Driver Release"
#define SUPPORT_CHIP " SM750, SM768"

#define _version_ "1.3.1.1"

#undef NO_WC
#undef HW_I2C
#undef NEED_PRIME_EXPORT

#ifdef UNUSED
#elif defined(__GNUC__)
#define UNUSED(x) UNUSED_##x __attribute__((unused))
#elif defined(__LCLINT__)
#define UNUSED(x) /*@unused@*/ x
#elif defined(__cplusplus)
#define UNUSED(x)
#else
#define UNUSED(x) x
#endif

#define SMI_MAX_FB_HEIGHT 3840 //8192
#define SMI_MAX_FB_WIDTH 3840 //8192

#define smi_DPMS_CLEARED (-1)

extern int g_specId;
extern int smi_pat;
extern int smi_bpp;
extern int force_connect;
extern int lvds_channel;

struct smi_750_register;
struct smi_768_register;

#if RHEL_VERSION_LOWER_THAN(8, 2)
#define DRM_FILE_PAGE_OFFSET (0x100000000ULL >> PAGE_SHIFT)
#endif

#if (RHEL_VERSION_HIGHER_THAN(7, 7) && RHEL_VERSION_LOWER_THAN(8, 0)) ||                           \
	RHEL_VERSION_HIGHER_THAN(8, 1)
#if RHEL_VERSION_LOWER_THAN(8, 2)
struct smi_bo;
#endif

struct smi_cursor {
#if RHEL_VERSION_HIGHER_THAN(8, 2)
	struct drm_gem_vram_object *cursors[2];
#else
	struct smi_bo *cursors[2];
#endif
	unsigned int next_index;
};
#endif

struct smi_device {
	struct drm_device *dev;
	unsigned long flags;

	resource_size_t rmmio_base;
	resource_size_t rmmio_size;
	void __iomem *rmmio;

	struct smi_mc mc;
	struct smi_mode_info mode_info;

#if (RHEL_VERSION_HIGHER_THAN(7, 7) && RHEL_VERSION_LOWER_THAN(8, 0)) ||                           \
	RHEL_VERSION_HIGHER_THAN(8, 1)
	struct smi_cursor cursor;
#endif

	int num_crtc;
	int fb_mtrr;
	bool need_dma32;
#if RHEL_VERSION_LOWER_THAN(8, 2)
	struct {
#if RHEL_VERSION_LOWER_THAN(7, 7) ||                                                               \
	(RHEL_VERSION_HIGHER_THAN(8, 0) && RHEL_VERSION_LOWER_THAN(8, 1))
		struct drm_global_reference mem_global_ref;
		struct ttm_bo_global_ref bo_global_ref;
#endif
		struct ttm_bo_device bdev;
	} ttm;
#endif
	bool mm_inited;
	union {
		struct smi_750_register *regsave;
		struct smi_768_register *regsave_768;
	};
	struct edid dvi_edid[2];
	struct edid vga_edid[2];
	struct edid hdmi_edid[2];
	struct drm_display_mode *fixed_mode;
	bool is_hdmi;
	bool is_boot_gpu;
};

struct smi_encoder {
	struct drm_encoder base;
	int last_dpms;
};

struct smi_connector {
	struct drm_connector base;
};

/* smi_main.c */
int smi_device_init(struct smi_device *cdev, struct drm_device *ddev, struct pci_dev *pdev,
		    uint32_t flags);
void smi_device_fini(struct smi_device *cdev);
int smi_driver_load(struct drm_device *dev, unsigned long flags);
#if RHEL_VERSION_HIGHER_THAN(7, 5)
void smi_driver_unload(struct drm_device *dev);
#else
int smi_driver_unload(struct drm_device *dev);
#endif
#if RHEL_VERSION_LOWER_THAN(7, 3)
int smi_gem_init_object(struct drm_gem_object *obj);
#endif
int smi_dumb_create(struct drm_file *file, struct drm_device *dev,
		    struct drm_mode_create_dumb *args);
#if RHEL_VERSION_LOWER_THAN(8, 2)
int smi_framebuffer_init(struct drm_device *dev, struct smi_framebuffer *gfb,
			 const struct drm_mode_fb_cmd2 *mode_cmd, struct drm_gem_object *obj);
int smi_gem_create(struct drm_device *dev, u32 size, bool iskernel, struct drm_gem_object **obj);
int smi_dumb_mmap_offset(struct drm_file *file, struct drm_device *dev, uint32_t handle,
			 uint64_t *offset);
int smi_dumb_destroy(struct drm_file *file, struct drm_device *dev, uint32_t handle);
void smi_gem_free_object(struct drm_gem_object *obj);
#endif

/* smi_plane.c */
#if RHEL_VERSION_LOWER_THAN(7, 4)
struct drm_plane *smi_plane_init(struct smi_device *cdev, unsigned int possible_crtcs);
#else
struct drm_plane *smi_plane_init(struct smi_device *cdev, unsigned int possible_crtcs,
				 enum drm_plane_type type);
#endif

/* smi_mode.c */
int smi_modeset_init(struct smi_device *cdev);
void smi_modeset_fini(struct smi_device *cdev);
#define to_smi_crtc(x) container_of(x, struct smi_crtc, base)
#define to_smi_encoder(x) container_of(x, struct smi_encoder, base)

/* smi_fbdev.c */
#if RHEL_VERSION_LOWER_THAN(8, 2)
int smi_fbdev_init(struct smi_device *cdev);
void smi_fbdev_fini(struct smi_device *cdev);
void smi_fb_zfill(struct drm_device *dev, struct smi_fbdev *gfbdev);
void smi_fb_output_poll_changed(struct smi_device *sdev);
#define to_smi_framebuffer(x) container_of(x, struct smi_framebuffer, base)
#endif

/* smi_ttm.c */
#if RHEL_VERSION_LOWER_THAN(8, 2)
struct smi_bo {
	struct ttm_buffer_object bo;
	struct ttm_placement placement;
	struct ttm_bo_kmap_obj kmap;
	struct drm_gem_object gem;
#if RHEL_VERSION_HIGHER_THAN(7, 3)
	struct ttm_place placements[3];
#else
	u32 placements[3];
#endif
	int pin_count;
	struct ttm_bo_kmap_obj dma_buf_vmap;
};
#define gem_to_smi_bo(gobj) container_of((gobj), struct smi_bo, gem)

static inline struct smi_bo *smi_bo(struct ttm_buffer_object *bo)
{
	return container_of(bo, struct smi_bo, bo);
}
#if RHEL_VERSION_LOWER_THAN(7, 3)
int smi_bo_create(struct drm_device *dev, int size, int align, uint32_t flags, struct sg_table *sg,
		  struct smi_bo **psmibo);
#else
int smi_bo_create(struct drm_device *dev, int size, int align, uint32_t flags, struct sg_table *sg,
		  struct reservation_object *resv, struct smi_bo **psmibo);
#endif
void smi_ttm_placement(struct smi_bo *bo, int domain);
void smi_bo_ttm_destroy(struct ttm_buffer_object *tbo);
int smi_mmap(struct file *filp, struct vm_area_struct *vma);
int smi_bo_pin(struct smi_bo *bo, u32 pl_flag, u64 *gpu_addr);
int smi_bo_unpin(struct smi_bo *bo);

static inline int smi_bo_reserve(struct smi_bo *bo, bool no_wait)
{
	int ret;
#if RHEL_VERSION_LOWER_THAN(7, 4)
	ret = ttm_bo_reserve(&bo->bo, true, no_wait, false, NULL);
#else
	ret = ttm_bo_reserve(&bo->bo, true, no_wait, NULL);
#endif
	if (ret) {
		if (ret != -ERESTARTSYS && ret != -EBUSY)
			DRM_ERROR("reserve failed %p\n", bo);
		return ret;
	}
	return 0;
}

static inline void smi_bo_unreserve(struct smi_bo *bo)
{
	ttm_bo_unreserve(&bo->bo);
}
#endif
int smi_mm_init(struct smi_device *smi);
void smi_mm_fini(struct smi_device *smi);

/* smi_prime.c */
//struct sg_table *smi_gem_prime_get_sg_table(struct drm_gem_object *obj);
struct drm_gem_object *smi_gem_prime_import_sg_table(struct drm_device *dev,
						     struct dma_buf_attachment *attach,
						     struct sg_table *sg);
#if RHEL_VERSION_LOWER_THAN(8, 2)
void *smi_gem_prime_vmap(struct drm_gem_object *obj);
void smi_gem_prime_vunmap(struct drm_gem_object *obj, void *vaddr);
int smi_gem_prime_pin(struct drm_gem_object *obj);
void smi_gem_prime_unpin(struct drm_gem_object *obj);
#endif

/* smi_cursor.c */
#if (RHEL_VERSION_HIGHER_THAN(7, 7) && RHEL_VERSION_LOWER_THAN(8, 0)) ||                           \
	RHEL_VERSION_HIGHER_THAN(8, 1)
int smi_cursor_init(struct smi_device *sdev);
void smi_cursor_fini(struct smi_device *sdev);
int smi_crtc_cursor_set(struct drm_crtc *crtc, struct drm_file *file_priv, unsigned int handle,
			unsigned int width, unsigned int height);
int smi_crtc_cursor_move(struct drm_crtc *crtc, int x, int y);
#endif

/* smi_snd.c */
int smi_audio_init(struct smi_device *dev);
void smi_audio_remove(struct smi_device *dev);

#define MAX_CRTC 2
#define MAX_ENCODER 3

#define smi_LUT_SIZE 256
#define PALETTE_INDEX 0x8
#define PALETTE_DATA 0x9

#define USE_DVI 1
#define USE_VGA (1 << 1)
#define USE_HDMI (1 << 2)
#define USE_DVI_VGA (USE_DVI | USE_VGA)
#define USE_DVI_HDMI (USE_DVI | USE_HDMI)
#define USE_VGA_HDMI (USE_VGA | USE_HDMI)
#define USE_ALL (USE_DVI | USE_VGA | USE_HDMI)

/* please use revision id to distinguish sm750le and sm750*/
#define SPC_SM750 0
#define SPC_SM712 1
#define SPC_SM502 2
#define SPC_SM768 3
//#define SPC_SM750LE 8

#define PCI_VENDOR_ID_SMI 0x126f
#define PCI_DEVID_LYNX_EXP 0x0750
#define PCI_DEVID_SM768 0x0768

#define BPP32_RED 0x00ff0000
#define BPP32_GREEN 0x0000ff00
#define BPP32_BLUE 0x000000ff
#define BPP32_WHITE 0x00ffffff
#define BPP32_GRAY 0x00808080
#define BPP32_YELLOW 0x00ffff00
#define BPP32_CYAN 0x0000ffff
#define BPP32_PINK 0x00ff00ff
#define BPP32_BLACK 0x00000000

#define BPP16_RED 0x0000f800
#define BPP16_GREEN 0x000007e0
#define BPP16_BLUE 0x0000001f
#define BPP16_WHITE 0x0000ffff
#define BPP16_GRAY 0x00008410
#define BPP16_YELLOW 0x0000ffe0
#define BPP16_CYAN 0x000007ff
#define BPP16_PINK 0x0000f81f
#define BPP16_BLACK 0x00000000

#endif /* __SMI_DRV_H__ */
